import os
from pathlib import Path
from colorama import Fore, Back
import art

APP_NAME = 'termy'
TERMY_DIR = Path.home() / APP_NAME
TERMY_COMMANDS_FILE = TERMY_DIR / 'commands.csv'
SCOPES = ['https://www.googleapis.com/auth/spreadsheets.readonly']
app_root = os.path.dirname(__file__)
CREDENTIALS_JSON = os.path.join(app_root, 'resources/credentials.json')
TOKEN_JSON = os.path.join(app_root, 'resources/tokens.json')
CREDS_OBJECT_FILE = os.path.join(app_root, "resources/creds.pkl")
CONFIG = os.path.join(app_root, "resources/config.json")
GPT3_CONFIG = os.path.join(app_root, "resources/gpt3_config.json")
SHEET_NAME = "Sheet1"

VERSION = '0.0.8'

MATCH_THRESHOLD = 60

TERMY_CONFIGURE_MESSAGE = Fore.RED + "Termy is not yet configured \n" + Fore.YELLOW + \
                          "Please configure it using " + Fore.GREEN + "termy --configure"
SERVER_ERROR = Fore.RED + "Unable to find the server at sheets.googleapis.com. Please check your connection!"
HTPP_SHEET_ERROR = Fore.RED + "Sheet could not be parsed. Please check if Sheet ID and Sheet Name is correct"
TERMY_BANNER = art.text2art("Termy")
TERMY_INTRO_MESSAGE = Fore.LIGHTYELLOW_EX + f'''
{TERMY_BANNER}
Termy is a terminal assistant which is focussed on easing out the developers life
by triggering commands just based on the Natural language.

To get started:
Configure termy using ''' + Fore.LIGHTGREEN_EX + '''termy --configure .''' + Fore.LIGHTYELLOW_EX + '''This will connect termy to your google sheet containg commands.
Once configuration is done, you can search using''' + Fore.LIGHTGREEN_EX + ''' termy <search_text> ''' + \
                      Fore.LIGHTYELLOW_EX + '''\nYou can also visit https://pypi.org/project/termy/ for more details \n\nIf you have any feedbacks on this tool, please use this form : https://forms.gle/8sHNPD9PNyVupFht8
You can also visit our github page and raise issues there. : https://github.com/dingusagar/termy
'''
AUTHENTICATE_ERROR = "Failed to authenticate google sheets integration"

SHEET_ID_INPUT_MESSAGE = f'''{Fore.LIGHTCYAN_EX}
Termy needs the sheet id of your google sheet link.

How to find your sheet id ?
Sheet id for the following sample sheet is the highlighted part : https://docs.google.com/spreadsheets/d/{Back.WHITE}{Fore.BLACK}1-wtkODsgt0EJzARAo7BBNOXwkd1W3vDiPH1HZoyskXI{Back.RESET}{Fore.LIGHTCYAN_EX}/edit#gid=0  

Enter the Sheet ID for your google sheet : {Fore.RESET}'''

SHEET_LINK_INPUT = f'''{Fore.LIGHTCYAN_EX}
Termy needs the google sheet link where the commands and queries are stored.

Your google sheet should follow the same format as this sample sheet : https://docs.google.com/spreadsheets/d/1-wtkODsgt0EJzARAo7BBNOXwkd1W3vDiPH1HZoyskXI/edit#gid=0  

Paste your google sheet link here : {Fore.RESET}'''

INVALID_SHEET_LINK = f'''{Fore.RED}Invalid Sheet Link.
Google sheet link should look something like this : https://docs.google.com/spreadsheets/d/1-wtkODsgt0EJzARAo7BBNOXwkd1W3vDiPH1HZoyskXI/edit#gid=0
Try again with the proper sheet link. {Fore.RESET}'''

GPT3_API_KEY_INPUT = f'''{Fore.LIGHTCYAN_EX}
Need API Key for querying to GPT-3. If you dont have one, sign up for an account and get it by clicking the link here https://beta.openai.com/account/api-keys 

API KEY :{Fore.RESET} '''


STOPWORDS = {'plz', 'pls', 'want', 'can', 'you', 'get', 'check', 'the', 'a', 'able', 'about', 'above', 'abst',
             'accordance', 'according', 'accordingly', 'across', 'act', 'actually', 'added', 'adj', 'affected',
             'affecting', 'affects', 'after', 'afterwards', 'again', 'against', 'ah', 'all', 'almost', 'alone', 'along',
             'already', 'also', 'although', 'always', 'am', 'among', 'amongst', 'an', 'and', 'announce', 'another',
             'any', 'anybody', 'anyhow', 'anymore', 'anyone', 'anything', 'anyway', 'anyways', 'anywhere', 'apparently',
             'approximately', 'are', 'aren', 'arent', 'arise', 'around', 'as', 'aside', 'ask', 'asking', 'at', 'auth',
             'available', 'away', 'awfully', 'b', 'back', 'be', 'became', 'because', 'become', 'becomes', 'becoming',
             'been', 'before', 'beforehand', 'begin', 'beginning', 'beginnings', 'begins', 'behind', 'being', 'believe',
             'below', 'beside', 'besides', 'between', 'beyond', 'biol', 'both', 'brief', 'briefly', 'but', 'by', 'c',
             'ca', 'came', 'can', 'cannot', "can't", 'cause', 'causes', 'certain', 'certainly', 'co', 'com', 'come',
             'comes', 'contain', 'containing', 'contains', 'could', 'couldnt', 'd', 'date', 'did', "didn't",
             'different', 'do', 'does', "doesn't", 'doing', 'done', "don't", 'down', 'downwards', 'due', 'during', 'e',
             'each', 'ed', 'edu', 'effect', 'eg', 'eight', 'eighty', 'either', 'else', 'elsewhere', 'end', 'ending',
             'enough', 'especially', 'et', 'et-al', 'etc', 'even', 'ever', 'every', 'everybody', 'everyone',
             'everything', 'everywhere', 'ex', 'except', 'f', 'far', 'few', 'ff', 'fifth', 'first', 'five', 'fix',
             'followed', 'following', 'follows', 'for', 'former', 'formerly', 'forth', 'found', 'four', 'from',
             'further', 'furthermore', 'g', 'gave', 'get', 'gets', 'getting', 'give', 'given', 'gives', 'giving', 'go',
             'goes', 'gone', 'got', 'gotten', 'h', 'had', 'happens', 'hardly', 'has', "hasn't", 'have', "haven't",
             'having', 'he', 'hed', 'hence', 'her', 'here', 'hereafter', 'hereby', 'herein', 'heres', 'hereupon',
             'hers', 'herself', 'hes', 'hi', 'hid', 'him', 'himself', 'his', 'hither', 'home', 'how', 'howbeit',
             'however', 'hundred', 'i', 'id', 'ie', 'if', "i'll", 'im', 'immediate', 'immediately', 'importance',
             'important', 'in', 'inc', 'indeed', 'index', 'information', 'instead', 'into', 'invention', 'inward', 'is',
             "isn't", 'it', 'itd', "it'll", 'its', 'itself', "i've", 'j', 'just', 'k', 'keep', 'keeps', 'kept', 'kg',
             'km', 'know', 'known', 'knows', 'l', 'largely', 'last', 'lately', 'later', 'latter', 'latterly', 'least',
             'less', 'lest', 'let', 'lets', 'like', 'liked', 'likely', 'line', 'little', "'ll", 'look', 'looking',
             'looks', 'ltd', 'm', 'made', 'mainly', 'make', 'makes', 'many', 'may', 'maybe', 'me', 'mean', 'means',
             'meantime', 'meanwhile', 'merely', 'mg', 'might', 'million', 'miss', 'ml', 'more', 'moreover', 'most',
             'mostly', 'mr', 'mrs', 'much', 'mug', 'must', 'my', 'myself', 'n', 'na', 'name', 'namely', 'nay', 'nd',
             'near', 'nearly', 'necessarily', 'necessary', 'need', 'needs', 'neither', 'never', 'nevertheless', 'new',
             'next', 'nine', 'ninety', 'no', 'nobody', 'non', 'none', 'nonetheless', 'noone', 'nor', 'normally', 'nos',
             'not', 'noted', 'nothing', 'now', 'nowhere', 'o', 'obtain', 'obtained', 'obviously', 'of', 'off', 'often',
             'oh', 'ok', 'okay', 'old', 'omitted', 'on', 'once', 'one', 'ones', 'only', 'onto', 'or', 'ord', 'other',
             'others', 'otherwise', 'ought', 'our', 'ours', 'ourselves', 'out', 'outside', 'over', 'overall', 'owing',
             'own', 'p', 'page', 'pages', 'part', 'particular', 'particularly', 'past', 'per', 'perhaps', 'placed',
             'please', 'plus', 'poorly', 'possible', 'possibly', 'potentially', 'pp', 'predominantly', 'present',
             'previously', 'primarily', 'probably', 'promptly', 'proud', 'provides', 'put', 'q', 'que', 'quickly',
             'quite', 'qv', 'r', 'ran', 'rather', 'rd', 're', 'readily', 'really', 'recent', 'recently', 'ref', 'refs',
             'regarding', 'regardless', 'regards', 'related', 'relatively', 'research', 'respectively', 'resulted',
             'resulting', 'results', 'right', 'run', 's', 'said', 'same', 'saw', 'say', 'saying', 'says', 'sec',
             'section', 'see', 'seeing', 'seem', 'seemed', 'seeming', 'seems', 'seen', 'self', 'selves', 'sent',
             'seven', 'several', 'shall', 'she', 'shed', "she'll", 'shes', 'should', "shouldn't", 'show', 'showed',
             'shown', 'showns', 'shows', 'significant', 'significantly', 'similar', 'similarly', 'since', 'six',
             'slightly', 'so', 'some', 'somebody', 'somehow', 'someone', 'somethan', 'something', 'sometime',
             'sometimes', 'somewhat', 'somewhere', 'soon', 'sorry', 'specifically', 'specified', 'specify',
             'specifying', 'still', 'stop', 'strongly', 'sub', 'substantially', 'successfully', 'such', 'sufficiently',
             'suggest', 'sup', 'sure'}



import os.path

from colorama import Fore
from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow

from termy.constants import TOKEN_JSON, SCOPES, AUTHENTICATE_ERROR
from termy.utils.utils import log_s5864, apply_color_and_rest


def google_auth():
    try:
        creds = None
        # os.remove(TOKEN_JSON)
        if os.path.exists(TOKEN_JSON):
            creds = Credentials.from_authorized_user_file(TOKEN_JSON, SCOPES)
        if not creds or not creds.valid:
            if creds and creds.expired and creds.refresh_token:
                creds.refresh(Request())
            else:
                flow = InstalledAppFlow.from_client_config(
                    log_s5864(), SCOPES)
                creds = flow.run_local_server(port=0)
            with open(TOKEN_JSON, 'w') as token:
                token.write(creds.to_json())

        return creds
    except Exception as e:
        print(apply_color_and_rest(Fore.RED, AUTHENTICATE_ERROR), e)

def google_auth_renew():
    try:
        os.remove(TOKEN_JSON) if os.path.exists(TOKEN_JSON) else None
        flow = InstalledAppFlow.from_client_config(
            log_s5864(), SCOPES)
        creds = flow.run_local_server(port=0)
        with open(TOKEN_JSON, 'w') as token:
            token.write(creds.to_json())
        return creds
    except Exception as e:
        print(apply_color_and_rest(Fore.RED, AUTHENTICATE_ERROR), e)




import json
import sqlite3
import sys

import openai
from colorama import Fore

from termy import TERMY_DIR
from termy.constants import GPT3_API_KEY_INPUT, GPT3_CONFIG
from termy.utils import apply_color_and_rest


class GPT3TerminalClient:

    def __init__(self):
        self.API_KEY = self.load_api_key_from_config()
        if not self.API_KEY:
            self.API_KEY = input(GPT3_API_KEY_INPUT)

        if not self.API_KEY:
            sys.exit(apply_color_and_rest(Fore.RED, "No API KEY was provided. Exiting."))

        self.save_api_key_in_config()
        self.platform = self.get_platform()
        self.initDB()

    def initDB(self):
        self.cache = sqlite3.connect(TERMY_DIR / ".cbot_cache")
        self.cache.execute("""
                       CREATE TABLE IF NOT EXISTS questions 
                       (id INTEGER PRIMARY KEY,
                       question TEXT,
                       answer TEXT,
                       count INTEGER DEFAULT 1)""")

    def closeDB(self):
        self.cache.commit()
        self.cache.close()

    def checkQ(self, question_text):
        sql = "SELECT id,answer,count FROM questions WHERE question =" + question_text
        answer = self.cache.execute("SELECT id,answer,count FROM questions WHERE question = ?", (question_text,))
        answer = answer.fetchone()
        if (answer):
            response = answer[1]
            newcount = int(answer[2]) + 1
            counter = self.cache.execute(" UPDATE questions SET count = ? WHERE id = ?", (newcount, answer[0]))
            return (response)
        else:
            return (False)

    def insertQ(self, question_text, answer_text):
        answer = self.cache.execute("DELETE FROM questions WHERE question = ?", (question_text,))
        answer = self.cache.execute("INSERT INTO questions (question,answer) VALUES (?,?)", (question_text, answer_text))


    def get_command(self, question):
        if not question:
            sys.exit(apply_color_and_rest(Fore.RED,
                                          'Empty query. Give a query to get the command. Eg: termy --gpt "How do I duplicate a folder" '))

        cache_answer = self.checkQ(question)
        if cache_answer:
            return cache_answer

        prompt = "I am a command line translation tool for " + self.platform + "."
        prompt = prompt + """
        Ask me what you want to do and I will tell you how to do it in a unix command.
        Q: How do I copy a file
        cp filename.txt destination_filename.txt
        Q: How do I duplicate a folder?
        cp -a source_folder/ destination_folder/
        Q: How do display a calendar?
        cal
        Q: how do I convert a .heic file to jpg?
        convert source.heic destination.jpg
        Q: navigate to my desktop
        cd ~/Desktop/
        Q: How do I shutdown the computer?
        sudo shutdown -h now
        """

        temp_question = question
        if not ("?" in question):
            temp_question = question + "?"

        prompt = prompt + "Q: " + temp_question + "\n"
        openai.api_key = self.API_KEY
        response = openai.Completion.create(
            engine="davinci",
            prompt=prompt,
            temperature=0,
            max_tokens=100,
            top_p=1,
            frequency_penalty=0,
            presence_penalty=0,
            stop=["\n"]
        )

        result = response['choices'][0]['text']
        self.insertQ(question, result)
        self.closeDB()
        return result.strip()

    def load_api_key_from_config(self):
        with open(GPT3_CONFIG, 'r') as file:
            config = json.load(file)
        return config.get('GPT3_API_KEY', None)

    def get_platform(self):
        platform = sys.platform
        if platform == "darwin":
            platform = "Mac"
        elif platform == "win32":
            platform = "Windows"
        else:
            platform = "Linux"
        return platform

    def save_api_key_in_config(self):
        with open(GPT3_CONFIG, 'w') as f:
            json.dump({'GPT3_API_KEY' : self.API_KEY}, f)



import json
import os
import pickle
import sys

import pandas as pd
from colorama import Fore
from rapidfuzz import process, fuzz

from termy.constants import TERMY_COMMANDS_FILE, MATCH_THRESHOLD, CREDS_OBJECT_FILE, CONFIG, SHEET_NAME, \
    TERMY_CONFIGURE_MESSAGE, SHEET_LINK_INPUT, INVALID_SHEET_LINK, STOPWORDS
from termy.service.aunthenticator.authenticate import google_auth_renew
from termy.service.content_extractor.get_sheet_content import get_sheet_content_into_csv
from termy.service.gpt_client.gpt3_terminal_client import GPT3TerminalClient
from termy.utils import save_object, apply_color_and_rest

creds = None
sheet_id = None
sheet_name = None


def configure_termy():
    global creds
    global sheet_name
    global sheet_id
    sheet_link = input(SHEET_LINK_INPUT)
    link_parts = sheet_link.split('/')
    if len(link_parts) >= 5:
        sheet_id = link_parts[5]
    else:
        sys.exit(INVALID_SHEET_LINK)
    sheet_name = input(apply_color_and_rest(Fore.LIGHTCYAN_EX,
                                            "Please enter the Sheet Name for the google sheet that "
                                            "contains the commands data (Press enter for default : 'Sheet1'): "))
    if not sheet_name:
        sheet_name = 'Sheet1'
    config = {"sheet_id": sheet_id, "sheet_name": sheet_name}
    with open(CONFIG, 'w') as f:
        json.dump(config, f)
    print(apply_color_and_rest(Fore.LIGHTGREEN_EX, f'Configuring Termy...'))
    creds = google_auth_renew()
    save_object(creds, CREDS_OBJECT_FILE)
    update_termy()


def update_termy():
    try:
        with open(CREDS_OBJECT_FILE, 'rb') as config_dictionary_file:
            creds = pickle.load(config_dictionary_file)
        with open(CONFIG, 'r') as f:
            config = json.load(f)
        get_sheet_content_into_csv(config.get("sheet_id"), config.get("sheet_name", SHEET_NAME), creds)
    except FileNotFoundError as e:
        sys.exit(TERMY_CONFIGURE_MESSAGE)


def execute_command(command):
    print(apply_color_and_rest(Fore.LIGHTCYAN_EX, f'Detected Command : {command}'))
    user_input = input(Fore.LIGHTCYAN_EX + f'Press enter to execute' + Fore.RESET)
    if user_input in ['n', 'no', 'c', 'cancel']:
        print(Fore.RED + 'Abort..')
    else:
        os.system(command)

def remove_stopwords(query):
    tokens = query.split()
    tokens = [token for token in tokens if not token in STOPWORDS]
    return ' '.join(tokens)


def rank_matches(matches, search_text):
    highest_score = matches[0][1]
    if highest_score < MATCH_THRESHOLD: # low threshold, skip reranking
        return matches[0]

    # reranking logic based on min number of tokens not matched.
    tokens_not_matched_counts = []
    for match in matches[:10]:
        match_score = match[1]
        if match_score != highest_score:
            break
        matched_query = match[0]
        tokens_not_matched_count = 0
        for token in matched_query.split():
            if token not in search_text:
                tokens_not_matched_count += 1
        tokens_not_matched_counts.append(tokens_not_matched_count)

    best_match_index = tokens_not_matched_counts.index(min(tokens_not_matched_counts))
    return matches[best_match_index]


def search_and_execute(search_text):
    commands, queries = load_commands_and_queries()
    search_text = remove_stopwords(search_text)
    matches = process.extract(search_text, queries, scorer=fuzz.token_set_ratio)
    if matches:
        match = rank_matches(matches, search_text)
        match_query, score, index = match
        if score >= MATCH_THRESHOLD:
            command = commands[index]
            execute_command(command)
        else:
            print(apply_color_and_rest(Fore.RED, 'No match found :('))
            print(apply_color_and_rest(Fore.RED,
                                       'Try adding the query and command to the Sheet and do'
                                       + Fore.GREEN + ' termy --update'))
    else:
        print(apply_color_and_rest(Fore.RED, 'No match found :('))
        print(apply_color_and_rest(Fore.RED,
                                   'Try adding the query and command to the Sheet and do '
                                   + Fore.GREEN + ' termy --update'))


def show_configs():
    print('show configs not yet supported')


def load_commands_and_queries():
    try:
        df = pd.read_csv(TERMY_COMMANDS_FILE)
        commands, queries = list(df['commands']), list(df['query'])
        final_commands, final_queries = [], []
        for i, query in enumerate(queries):
            query_variations = query.split('\n')
            query_variations = [variation for variation in query_variations if variation]
            final_queries.extend(query_variations)
            final_commands.extend([commands[i]] * len(query_variations))
        return final_commands, final_queries
    except FileNotFoundError as e:
        sys.exit(TERMY_CONFIGURE_MESSAGE)


def resolve_command_from_GPT3(query):
    client = GPT3TerminalClient()
    command = client.get_command(query)
    if command:
        execute_command(command)
    else:
        print(apply_color_and_rest(Fore.RED, 'No match found :('))



if __name__ == '__main__':
    configure_termy()



import argparse
import sys

from termy.constants import TERMY_INTRO_MESSAGE, VERSION
from termy.service.flow_handler.handle_flows import configure_termy, search_and_execute, update_termy, \
    resolve_command_from_GPT3

DESCRIPTION = TERMY_INTRO_MESSAGE


def init_cli_app():
    parser = argparse.ArgumentParser(add_help=False,
                                     description=DESCRIPTION, formatter_class=argparse.RawTextHelpFormatter)

    parser.add_argument('search', help='Input String', nargs='*')
    parser.add_argument("-c", "--configure", action='store_true', help="Configure your termy")
    parser.add_argument("--show-config", action='store_true', help="Shows the current configurations")
    parser.add_argument("--gpt3", action='store_true', help="Give the query in natural language to get terminal command from GPT-3 API")
    parser.add_argument("-u", "--update", action='store_true',
                        help="Update termy to be updated with the latest commands")
    parser.add_argument("-v", "--version", action='store_true',
                        help="Version Info")
    args = parser.parse_args()

    if args.gpt3: # gpt3 search query
        if args.search:
            query = ' '.join(args.search)
        else:
            query = ''
        resolve_command_from_GPT3(query)
    elif args.search: # regular search query
        query = ' '.join(args.search)
        search_and_execute(query)
    elif args.configure:
        configure_termy()
    elif args.update:
        update_termy()
    elif args.version:
        print(VERSION)
    elif args.gpt3:
        resolve_command_from_GPT3(args.gpt3)
    else:
        parser.print_help(sys.stdout)


if __name__ == '__main__':
    init_cli_app()
